""" Functionality on top of QApplication.
"""
import sys, traceback, Queue
from PyQt4.QtGui import *
import warnings

import srllib.threading
from srllib.signal import Signal
from _common import *
from srllib.qtgui import _signal

__all__ = ["Application", "get_app"]

class _AsyncEvent(QEvent):
    EventType = QEvent.User

    def __init__(self, func, obj, args, kwds):
        QEvent.__init__(self, self.__class__.EventType)
        self.func, self.obj, self.args, self.kwds = func, obj, args, kwds
        self.dispatch_event = srllib.threading.Event()

class Application(QApplication):
    """ Specialize QApplication to trap Python exceptions, inform the user and quit.
    @cvar the_app: The L{application<Application>} object, if instantiated
    (otherwise None).
    @ivar sig_exception: Emitted when detecting an unhandled exception.
    Parameters: Exception type, exception value, traceback, name of thread (None
    if main thread).
    """
    import srllib.signal
    sig_quitting = srllib.signal.Signal()
    the_app = None

    def __init__(self, argv=None, catch_exceptions=True):
        """
        @param catch_exceptions: Handle uncaught exceptions.
        """
        if argv is None:
            # sys.argv needn't be defined when embedding Python
            argv = getattr(sys, "argv", [])
        QApplication.__init__(self, argv)

        self.sig_exception = Signal()

        self.__hasQuit, self.__call_queue = False, []
        if catch_exceptions:
            sys.excepthook = self.__exchook
            srllib.threading.register_exceptionhandler(self.__thrdexc_hook)

        import PyQt4.QtGui
        PyQt4.QtGui.qApp = self
        Application.the_app = self

        self.__deferred_queue = Queue.Queue()
        timer = self.__timer = QTimer(self)
        QObject.connect(timer, SIGNAL("timeout()"), self.__slot_timed_out)
        timer.start(20)

    #{ Qt methods

    @staticmethod
    def setOverrideCursor(cursor):
        """ Set overriding cursor for application. Argument cursor should either
        be suitable enumeration or a QCursor.
        """
        if isinstance(cursor, int):
            cursor = QtGui.QCursor(cursor)
        QApplication.setOverrideCursor(cursor)

    def customEvent(self, e):
        if not isinstance(e, _AsyncEvent):
            return QApplication.customEvent(self, e)
        e.func(e.obj, *e.args, **e.kwds)
        e.dispatch_event.set()

    @classmethod
    def quit(cls):
        if cls.the_app.__hasQuit:
            warnings.warn("The app has already quit")
        cls.sig_quitting()
        QApplication.quit()
        cls.the_app.__hasQuit = True

    @classmethod
    def has_quit(cls):
        return cls.the_app.__hasQuit

    #}

    def queue_call(self, to_call, args=None, kwds=None):
        """ Queue a call for when control returns to the event loop.
        """
        args = args or ()
        kwds = kwds or {}
        self.__call_queue.append((to_call, args, kwds))
        QTimer.singleShot(0, self.__exec_call)

    def queue_deferred(self, mthd, args, kwds, optimize=False):
        """ Queue deferred method call to be dispatched in GUI thread.
        """
        self.__deferred_queue.put((mthd, args, kwds, optimize))

    def __slot_timed_out(self):
        """ Periodic callback for various chores.

        This callback is here used to dispatch background-thread signals, and
        as an opportunity for Python to process incoming OS signals (e.g.,
        SIGINT resulting from Ctrl+C).
        """
        # Find deferred calls and dispatch them

        to_dispatch = []
        while True:
            try: mthd, args, kwds, optimize = self.__deferred_queue.get_nowait()
            except Queue.Empty:
                break
            to_dispatch.append((mthd, args, kwds, optimize))

        last = None
        i = len(to_dispatch) - 1
        for mthd, args, kwds, optimize in reversed(to_dispatch):
            if mthd is last and optimize:
                # Don't call several times
                del to_dispatch[i]
            last = mthd
            i -= 1
        for mthd, args, kwds, optimize in to_dispatch:
            mthd(*args, **kwds)

    @_signal.deferred_slot
    def __thrdexc_hook(self, exc):
        self.__exchook(exc.exc_type, exc.exc_value, exc.exc_traceback,
                in_thread=exc.name)

    def __exchook(self, exc, value, tb, in_thread=None):
        self.sig_exception(exc, value, tb, in_thread)

        self.__timer.stop()
        # Don't act on Ctrl+C
        if not exc is KeyboardInterrupt:
            thrdSpecific = ""
            if in_thread:
                thrdSpecific = " in thread %s" % (in_thread,)

            msg = ' '.join(traceback.format_exception(exc, value, tb))
            message_critical("Fatal Error", "An unexpected exception was "
                    "encountered%s, the application will have to be shut "
                    "down." % (thrdSpecific,), detailed_text=msg,
                    informative_text="The detailed text provides full technical "
                    "information of how the error happened, so developers may "
                    "resolve the problem. This information should also be "
                    "visible in the application log.")

        if not self.__hasQuit:
            self.quit()
        self.processEvents()

    def __exec_call(self):
        toCall, args, kwds = self.__call_queue.pop(0)
        toCall(*args, **kwds)

_the_app = None

def get_app():
    """ Get the current L{Application} instance. """
    global _the_app
    if _the_app is not None:
        # Allow overriding of application, useful for tests
        return _the_app
    return Application.the_app



""" Signals/slots functionality. """
import types, weakref

from error import *

class _DeadReference(SrlError):
    pass

class _FunctionProxy(object):
    def __init__(self, func):
        self._ref = weakref.ref(func)

    def __call__(self, *args, **kwds):
        func = self._getRef()
        func(*args, **kwds)

    def __eq__(self, rhs):
        func = self._getRef()
        return func == rhs

    @property
    def object(self):
        return self._ref()

    def _getRef(self):
        func = self._ref()
        if func is None:
            raise _DeadReference
        return func

class CallFailure(SrlError):
    """ Failure to call slot. """

class _MethodProxy(object):
    def __init__(self, mthd):
        self._instanceRef, self._mthdRef = weakref.ref(mthd.im_self), weakref.ref(mthd.im_func)

    def __call__(self, *args, **kwds):
        instance, mthd = self._get_refs()
        try: mthd(instance, *args, **kwds)
        except TypeError, err:
            if not sys.exc_info()[-1].tb_next:
                # The exception happened in this frame
                raise CallFailure("Calling slot %s.%s resulted in TypeError, check \
your arguments; the original exception was: `%s'" %
                        (mthd.__module__, mthd.__name__, err.message,))
            raise

    def __eq__(self, rhs):
        if isinstance(rhs, _MethodProxy):
            return self._get_refs() == rhs._get_refs()

        if type(rhs) != types.MethodType:
            return False
        instance, mthd = self._get_refs()
        return instance == rhs.im_self and mthd == rhs.im_func

    @property
    def object(self):
        return self._instanceRef()

    def _get_refs(self):
        instance, mthd = self._instanceRef(), self._mthdRef()
        if instance is None:
            raise _DeadReference
        assert mthd is not None
        return instance, mthd

class Signal(object):
    """ A signal is a way to pass a message to interested observers.

    Observers connect methods ("slots") to the signal they're interested in, and
    when the signal is fired the methods are called back with the correct
    parameters.
    """
    __all_signals = set()

    def __init__(self):
        self._slots = []
        self.__obj2slots = {}
        Signal.__all_signals.add(self)
        self.__enabled = True

    def __call__(self, *args, **kwds):
        """ For each group in sorted order, call each slot.
        """
        if not self.__enabled:
            return

        for e in self._slots[:]:
            slot, defArgs, defKwds = e
            args += tuple(defArgs[len(args):])
            keywords = defKwds.copy()
            keywords.update(kwds)
            try:
                slot(*args, **keywords)
            except _DeadReference:
                self._slots.remove(e)

    def connect(self, slot, defArgs=(), defKwds={}):
        """ Connect this signal to a slot, alternatively grouped. Optionally,
        keywords can be bound to slot.
        """
        if isinstance(slot, types.MethodType):
            prxyTp = _MethodProxy
        else:
            prxyTp = _FunctionProxy
        self.__connect(prxyTp(slot), defArgs, defKwds)

    def disconnect(self, slot):
        """ Disconnect signal from slot.
        @raise ValueError: Not connected to slot.
        """
        e = self.__find_slot(slot)
        if e is None:
            raise ValueError(slot)
        self.__remove_entry(e)

    @classmethod
    def disconnect_all_signals(cls, obj):
        """ Disconnect all signals from an object and its methods.

        @note: If no connections are found for this object, no exception is
        raised.
        """
        for sig in cls.__all_signals:
            if sig.is_connected(obj):
                sig.disconnect_object(obj)

    def disconnect_object(self, obj):
        """ Disconnect an object and its methods. """
        objSlots = self.__obj2slots[obj]
        for s in objSlots:
            self.disconnect(s)

    def enable(self):
        self.__enabled = True

    def disable(self):
        self.__enabled = False

    def set_enabled(self, enabled):
        self.__enabled = enabled

    @property
    def is_enabled(self):
        return self.__enabled

    def is_connected(self, obj):
        """ Is an object connected to this signal. """
        return obj in self.__obj2slots

    def __connect(self, slot, defArgs, defKwds):
        e = (slot, defArgs, defKwds)
        if self.__find_slot(slot) is None:
            self._slots.append(e)

        if not slot.object in self.__obj2slots:
            self.__obj2slots[slot.object] = []
        self.__obj2slots[slot.object].append(slot)

    def __find_slot(self, slot):
        for e in self._slots[:]:
            try:
                if e[0] == slot:
                    return e
            except _DeadReference:
                self.__remove_entry(e)

    def __remove_entry(self, entry):
        self._slots.remove(entry)
        slot = entry[0]
        objSlots = self.__obj2slots[slot.object]
        objSlots.remove(slot)
        if not objSlots:
            del self.__obj2slots[slot.object]




import os.path, sys

from distribute_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

import srllib

extra_kwds = {}
setup(name="srllib",
        version=srllib.__version__,
        author="Arve Knudsen",
        author_email="arve.knudsen@gmail.com",
        url="http://code.google.com/p/srl-python-lib",
        license=srllib.__license__,
        description="General Python application library.",
        platforms=["All"],
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'Programming Language :: Python :: 2',
            'Topic :: Software Development :: Libraries',
            ],
        packages=find_packages(exclude=["Tests"]),
        **extra_kwds)



#
# (c) Dave Kirby 2001 - 2005
#     mock@thedeveloperscoach.com
# (c) Arve Knudsen 2007-2008
#     arve.knudsen@gmail.com
#
# Original call interceptor and call assertion code by Phil Dawes (pdawes@users.sourceforge.net)
# Call interceptor code enhanced by Bruce Cropley (cropleyb@yahoo.com.au)
#
# This Python  module and associated files are released under the FreeBSD
# license. Essentially, you can do what you like with it except pretend you wrote
# it yourself.
#
#
#     Copyright (c) 2005, Dave Kirby, 2007, Arve Knudsen
#
#     All rights reserved.
#
#     Redistribution and use in source and binary forms, with or without
#     modification, are permitted provided that the following conditions are met:
#
#         * Redistributions of source code must retain the above copyright
#           notice, this list of conditions and the following disclaimer.
#
#         * Redistributions in binary form must reproduce the above copyright
#           notice, this list of conditions and the following disclaimer in the
#           documentation and/or other materials provided with the distribution.
#
#         * Neither the name of this library nor the names of its
#           contributors may be used to endorse or promote products derived from
#           this software without specific prior written permission.
#
#     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#     ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
#     ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#     (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
#     ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#     (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#         mock@thedeveloperscoach.com


""" Mock object library for Python.

Mock objects can be used when unit testing
to remove a dependency on another production class. They are typically used
when the dependency would either pull in lots of other classes, or
significantly slow down the execution of the test.
They are also used to create exceptional conditions that cannot otherwise
be easily triggered in the class under test.
"""
__version__ = "0.2.0"

# Added in Python 2.1
import inspect
import re
import warnings

import srllib.inspect

class MockInterfaceError(Exception):
    pass

class Mock(object):
    """ The Mock class simulates any other class for testing purposes.

    All method calls are stored for later examination.
    @cvar mockInstances: Dictionary of all mock instances, indexed on class to
    discern different mock subclasses.
    @cvar _MockRealClass: For subclasses, indicate the class that is being
    mocked.
    """
    mockInstances = {}
    _MockRealClass = None

    def __init__(self, returnValues=None, properties=None, realClass=None,
            name=None, dontMock=[], attributes=None):
        """ Constructor.

        Methods that are not in the returnValues dictionary will return None.
        You may also supply a class whose interface is being mocked.  All calls
        will be checked to see if they appear in the original interface. Any
        calls to methods not appearing in the real class will raise a
        MockInterfaceError.  Any calls that would fail due to non-matching
        parameter lists will also raise a MockInterfaceError.  Both of these
        help to prevent the Mock class getting out of sync with the class it is
        Mocking.
        @param returnValues: Define return values for mocked methods.
        @param properties: Define return values for mocked properties.
        @param realClass: Specify the mocked class.
        @param name: Optionally specify mock's name.
        @param dontMock: Optionally a specify a set of methods to not mock.
        @param attributes: Define mocked attributes.
        @raise MockInterfaceError: An inconsistency was detected in the
        mock's interface.
        """
        if returnValues is None:
            returnValues = {}
        if properties is None:
            properties = {}
        if attributes is None:
            attributes = {}

        self.mockCalledMethods = {}
        self.mockAllCalledMethods = []
        self.mockReturnValues = returnValues
        self.mockReturnValuesArgs = {}
        self.mockExpectations = {}
        self.__realClassMethods = self.__realClassProperties = {}
        self.__name = name
        self.__methods = {}    # Keep a cache of methods
        self.__dontMock = dontMock
        self.__attributes = {}
        if realClass is None:
            realClass = self._MockRealClass
        self.__realClass = realClass
        if realClass is not None:
            # Verify interface versus mocked class
            if not inspect.isclass(realClass):
                raise TypeError(realClass)
            if issubclass(realClass, (MockCallable, Mock)):
                raise TypeError(realClass)
            # We treat all callable class members as methods
            self.__realClassMethods = srllib.inspect.get_members(realClass,
                    callable)

            # Verify that mocked methods exist in real class
            for retMethod in self.mockReturnValues.keys():
                if not self.__realClassMethods.has_key(retMethod):
                    raise MockInterfaceError("Return value supplied for method \
'%s' that was not in the original class (%s)" % (retMethod, realClass.__name__))

            # Verify that mocked properties exist in real class

            realprops = self.__realClassProperties = \
                srllib.inspect.get_members(realClass, inspect.isdatadescriptor)
            for name in properties:
                if  name not in realprops:
                    raise MockInterfaceError("'%s' is not a property of '%s'" %
                            (name, realClass))

            # Now properties
            mockprops = srllib.inspect.get_members(self.__class__,
                inspect.isdatadescriptor)
            for name, prop in mockprops.items():
                if name.startswith("mock"):
                    continue
                # Ignore certain special attributes
                if name in ["__weakref__"]:
                    continue
                if name not in realprops:
                    raise MockInterfaceError("'%s' is not a property of '%s'" %
                            (name, realClass))

            self.__realClassProperties = properties

        self.__setupSubclassMethodInterceptors()

        # Attributes
        for k, v in attributes.items():
            self.__attributes[k] = v

        # Record this instance among all mock instances
        tp = type(self)
        if not tp in Mock.mockInstances:
            Mock.mockInstances[tp] = []
        Mock.mockInstances[tp].append(self)

    def __str__(self):
        if self.__name is not None:
            return self.__name
        return object.__str__(self)

    def __getattr__(self, name):
        """ Override in order to mock class methods.

        This is called as the last resort, before an AttributeError would
        otherwise be raised.
        """
        try: return self.__dict__["_Mock__realClassProperties"][name]
        except KeyError: pass
        try: return self.__dict__["_Mock__attributes"][name]
        except KeyError: pass
        name_lower = name.lower()
        if name_lower.startswith("mock") or name_lower.startswith("_mock"):
            # Don't mock mock-methods!
            raise AttributeError(name)

        if (self.__realClass is not None and name not in self.__realClassMethods
            or name in self.__dontMock):
            raise AttributeError("%s: %s" % (self.__class__.__name__, name))

        # Keep a cache of methods for this object, so that references to the
        # mock's "methods" don't go out of scope
        return self.__methods.setdefault(name, MockCallable(name, self))

    def __call__(self, *args, **kwds):
        """ Allow calling directly. """
        return self.__getattr__("__call__")(*args, **kwds)

    @property
    def mockAccessedAttrs(self):
        """ All accessed attributes.
        """
        return self.__methods.keys()

    @classmethod
    def mockGetAllInstances(cls):
        """ Get all instances of this mock class.
        """
        return cls.mockInstances.get(cls, [])

    def mockClearCalls(self):
        """ Clear all calls registered so far. """
        self.mockAllCalledMethods = []
        self.mockCalledMethods.clear()

    def mockSetReturnValue(self, name, value):
        """ Set a return value for a method.
        """
        self.mockReturnValues[name] = value

    def mockSetReturnValueWithArgs(self, name, value, args=(), kwds={}):
        """ Set a return value for a method with certain arguments.
        """
        try: retVals = self.mockReturnValuesArgs[name]
        except KeyError: retVals = self.mockReturnValuesArgs[name] = []
        # Eliminiate eventual stale entry
        for i, (a, k, v) in enumerate(retVals[:]):
            if (a, k) == (args, kwds):
                del retVals[i]
        retVals.append((args, kwds, value))

    def mockAddReturnValues(self, **methodReturnValues ):
        self.mockReturnValues.update(methodReturnValues)

    def mockSetExpectation(self, name, testFn, after=0, until=0):
        """ Set an expectation for a method call. """
        self.mockExpectations.setdefault(name, []).append((testFn, after,
                until))

    def mockSetRaises(self, name, exc, until=None, after=None):
        """ Set an exception to be raised by a method.
        @param until: Optionally specify a call index until which the
        exception will be raised.
        @param after: Optionally specify a call index after which the exception
        will be raised.
        """
        mock_callable = self.__getattr__(name)
        assert isinstance(mock_callable, MockCallable), \
                "Expected MockCallable, got %r" % (mock_callable)
        mock_callable.setRaises(exc, until=until, after=after)

    def mockGetCall(self, idx):
        """ Get a certain L{call<MockCall>} that was made. """
        return self.mockAllCalledMethods[idx]

    def mockGetNamedCall(self, name, idx):
        """ Get a L{call<MockCall>} to a certain method that was made. """
        return self.mockGetNamedCalls(name)[idx]

    def mockGetAllCalls(self):
        """ Get all calls.
        @return: List of L{MockCall} objects, representing all the methods in
        the order they were called.
        """
        return self.mockAllCalledMethods

    def mockGetNamedCalls(self, methodName):
        """ Get all calls to a certain method.
        @return: List of L{MockCall} objects, representing all the calls to the
        named method in the order they werecalled.
        """
        return self.mockCalledMethods.get(methodName, [])

    def mockCheckCall(self, tester, index, name, *args, **kwargs):
        """ Test that the index-th call had the specified name and
        parameters.
        """
        try: call = self.mockAllCalledMethods[index]
        except IndexError:
            tester.fail("No call with index %d" % index)
        tester.assertEqual(name, call.name, "Expected call number %d to \
be to %s, but it was to %s instead" % (index, name, call.name,))
        call.checkArgs(tester, *args, **kwargs)

    def mockCheckCalls(self, tester, calls):
        """ Test that a specified sequence of calls were made.
        @param tester: The test case.
        @param calls: A sequence of (name, args, kwargs) tuples.
        """
        numCalls, numExpected = len(self.mockAllCalledMethods), len(calls)
        if numCalls != numExpected:
            if numCalls < numExpected:
                tester.fail("No more than %d calls were made (expected %d)" %
                        (numCalls, numExpected))
            else:
                tester.fail("%d calls were made, expected %d" %
                        (numCalls, numExpected))

        for i, call in enumerate(calls):
            name = call[0]
            try: args = call[1]
            except IndexError: args = ()
            try: kwds = call[2]
            except IndexError: kwds = {}
            self.mockCheckCall(tester, i, name, *args, **kwds)

    def mockCheckNamedCall(self, tester, methodName, index, *args, **kwargs):
        """ Test that the index-th call to a certain method had the specified
        parameters.
        @raise IndexError: No call with this index.
        """
        self.mockCalledMethods.get(methodName, [])
        try: call = self.mockCalledMethods.get(methodName, [])[index]
        except IndexError:
            raise ValueError("No call to %s with index %d" % (methodName, index))
        call.checkArgs(tester, *args, **kwargs)

    def mockCheckNamedCalls(self, tester, methodName, calls):
        """ Test that a specified sequence of calls to a certain method were
        made.
        @param tester: The test case.
        @param methodName: The method's name.
        @param calls: A sequence of (args, kwargs) tuples.
        """
        numCalls, numExpected = len(self.mockCalledMethods.get(methodName,
                [])), len(calls)
        if numCalls != numExpected:
            if numCalls < numExpected:
                tester.fail("No more than %d calls were made (expected %d)" %
                        (numCalls, numExpected))
            else:
                tester.fail("%d calls were made, expected %d" %
                        (numCalls, numExpected))

        for i, call in enumerate(calls):
            try: args = call[0]
            except IndexError: args = ()
            try: kwds = call[1]
            except IndexError: kwds = {}
            self.mockCheckNamedCall(tester, methodName, i, *args, **kwds)

    def __setupSubclassMethodInterceptors(self):
        """ Install MockCallables for subclass methods.
        """
        methods = srllib.inspect.get_members(self.__class__, inspect.isroutine)
        baseMethods = srllib.inspect.get_members(Mock, inspect.ismethod)
        # The other half of this pattern should match private methods, based on
        # the naming convention _<class name>__<method name>
        reIgnore = re.compile(r"(?:mock|_[^_]+__).+")
        for name in methods:
            # Filter methods of Mock base class, private methods and methods
            # that start with "mock"
            if (name not in baseMethods and not reIgnore.match(name)
                and name not in self.__dontMock):
                self.__dict__[name] = MockCallable(name, self, handcrafted=True)

    def _mockCheckInterfaceCall(self, name, callParams, callKwParams):
        """ Check that a call to a method of the given name to the original
        class with the given parameters would not fail.

        If it would fail, raise a MockInterfaceError.
        Based on the Python 2.3.3 Reference Manual section 5.3.4: Calls.
        """
        if self.__realClass is None:
            return
        if not self.__realClassMethods.has_key(name):
            raise MockInterfaceError("Calling mock method '%s' that was not \
found in the original class (%s)" % (name, self.__realClass.__name__))

        func = self.__realClassMethods[name]
        try:
            args, varargs, varkw, defaults = inspect.getargspec(func)
        except TypeError:
            # func is not a Python function. It is probably a builtin,
            # such as __repr__ or __coerce__. TODO: Checking?
            # For now assume params are OK.
            return

        # callParams doesn't include self; args does include self.
        numPosCallParams = 1 + len(callParams)

        if numPosCallParams > len(args) and not varargs:
            raise MockInterfaceError("Original %s() takes at most %s arguments \
(%s given)" %
                (name, len(args), numPosCallParams))

        # Get the number of positional arguments that appear in the call,
        # also check for duplicate parameters and unknown parameters
        numPosSeen = _getNumPosSeenAndCheck(numPosCallParams, callKwParams,
            args, varkw)

        lenArgsNoDefaults = len(args) - len(defaults or [])
        if numPosSeen < lenArgsNoDefaults:
            raise MockInterfaceError("Original %s() takes at least %s \
arguments (%s given)" % (name, lenArgsNoDefaults, numPosSeen))

try: import zope.interface
except ImportError: pass
else:
    from srllib.testing._ifacemock import InterfaceMock

def FunctionMock(returnValue=None, raises=None):
    """ Factory function for returning a mock acting as a function.
    @ivar returnValue: Optionally specify return value.
    @ivar raises: Optionally specify an exception this function shall raise.
    """
    fm = Mock(returnValues={"__call__": returnValue})
    if raises is not None:
        fm.mockSetRaises("__call__", raises)
    return fm

class MockFactory(object):
    """ Utility class intended to act as a fake constructor for mocks.

    Specified constructor arguments/keywords are used to actually construct the
    mock object, while the arguments/keywords received with the construction
    request are stored in the mock object in the attributes
    "mockConstructorArgs" and "mockConstructorKwds", respectively.

    If the object has a method "mockInit", that will be called after
    construction with the factory arguments/keywords.
    @ivar mock_constructed: Constructed objects.
    """
    def __init__(self, cls=Mock, mockArgs=(), mockKwds={}, directArgs=False):
        """ Constructor.

        @param cls: The class to create objects of.
        @param mockArgs: Arguments to pass to the class' constructor.
        @param mockKwds: Keyword arguments to pass to the class' constructor.
        @param directArgs: Use construction arguments directly when constructing
        the object?
        """
        self.mock_constructed = []
        (self.__cls, self.__args, self.__kwds, self.__directArgs) = (cls,
            mockArgs, mockKwds, directArgs)

    def __call__(self, *args, **kwds):
        if not self.__directArgs:
            obj = self.__cls(*self.__args, **self.__kwds)
        else:
            obj = self.__cls(*args, **kwds)
        obj.mockConstructorArgs, obj.mockConstructorKwds = (args, kwds)
        if hasattr(obj, "mockInit"):
            obj.mockInit(*args, **kwds)
        self.mock_constructed.append(obj)
        return obj
    
    @property
    def constructed(self):
        warnings.warn("use mock_constructed instead", DeprecationWarning, stacklevel=2)
        return self.mock_constructed

def _getNumPosSeenAndCheck(numPosCallParams, callKwParams, args, varkw):
    """
    Positional arguments can appear as call parameters either named as
    a named (keyword) parameter, or just as a value to be matched by
    position. Count the positional arguments that are given by either
    keyword or position, and check for duplicate specifications.
    Also check for arguments specified by keyword that do not appear
    in the method's parameter list.
    """
    posSeen = {}
    for arg in args[:numPosCallParams]:
        posSeen[arg] = True
    for kwp in callKwParams:
        if posSeen.has_key(kwp):
            raise MockInterfaceError("%s appears as both a positional and named \
parameter." % kwp)
        if kwp in args:
            posSeen[kwp] = True
        elif not varkw:
            raise MockInterfaceError("Original method does not have a parameter \
'%s'" % kwp)
    return len(posSeen)

class MockCall:
    """ MockCall records the name and parameters of a call to an instance
    of a Mock class.

    Instances of MockCall are created by the Mock class, but can be inspected
    later as part of the test.
    @ivar name: Name of callable.
    @ivar args: Arguments to callable.
    @ivar kwargs: Keyword arguments to callable.
    """
    def __init__(self, name, args, kwargs):
        self.name = name
        self.args = args
        self.kwargs = kwargs

    def checkArgs(self, tester, *args, **kwargs):
        args = tuple(args)
        assert isinstance(self.args, tuple), "%r" % (self.args,)
        tester.assertEqual(args, self.args, "Callable %s: Arguments %r differ "
            "from those expected: %r" % (self.name, self.args, args))
        tester.assertEqual(kwargs, self.kwargs, "Callable %s: Keyword arguments "
            "(%r) differ from those expected (%r)" % (self.name, self.kwargs,
                kwargs))

    def getParam(self, n):
        if isinstance(n, int):
            return self.args[n]
        elif isinstance(n, str):
            return self.kwargs[n]
        else:
            raise IndexError, 'illegal index type for getParam'

    @property
    def numArgs(self):
        return len(self.args)

    @property
    def numKwargs(self):
        return len(self.kwargs)

    #pretty-print the method call
    def __str__(self):
        s = self.name + "("
        sep = ''
        for p in self.args:
            s = s + sep + repr(p)
            sep = ', '
        items = self.kwargs.items()
        items.sort()
        for k,v in items:
            s = s + sep + k + '=' + repr(v)
            sep = ', '
        s = s + ')'
        return s
    def __repr__(self):
        return self.__str__()

class MockCallable:
    """ Intercept/record a call.

    The call is delegated to either the mock's dictionary of mock return
    values that was passed in to the constructor, or a handcrafted method
    of a Mock subclass.
    """
    def __init__(self, name, mock, handcrafted=False):
        """
        @param name: Name of callable.
        @param mock: Parent mock.
        @param handcrafted: ?
        """
        self.name = name
        self.mock = mock
        self.handcrafted = handcrafted
        self.__exc = None

    def __call__(self,  *params, **kwparams):
        self.mock._mockCheckInterfaceCall(self.name, params, kwparams)
        thisCall = self.recordCall(params,kwparams)
        self.checkExpectations(thisCall, params, kwparams)
        return self.makeCall(params, kwparams)

    def setRaises(self, exc, after=None, until=None):
        """ Set an exception that should be raised when called. """
        self.__exc = (exc, after, until)

    def recordCall(self, params, kwparams):
        """
        Record the MockCall in an ordered list of all calls, and an ordered
        list of calls for that method name.
        """
        thisCall = MockCall(self.name, params, kwparams)
        calls = self.mock.mockCalledMethods.setdefault(self.name, [])
        calls.append(thisCall)
        self.mock.mockAllCalledMethods.append(thisCall)
        return thisCall

    def makeCall(self, params, kwparams):
        if self.__exc is not None:
            exc, after, until = self.__exc
            # The call is recorded before it is made
            idx = len(self.mock.mockCalledMethods.get(self.name, [])) - 1
            if (after is None or idx > after) and (until is None or
                idx < until):
                raise exc

        if self.handcrafted and self.name not in self.mock.mockReturnValues:
            allPosParams = (self.mock,) + params
            func = _findFunc(self.mock.__class__, self.name)
            if not func:
                raise NotImplementedError
            return func(*allPosParams, **kwparams)
        else:
            # First see if there is a match for these specific call
            # parameters.
            for args, kwds, returnVal in self.mock.mockReturnValuesArgs.get(
                self.name, []):
                if args == params and kwds == kwparams:
                    break
            else:
                # Go for the generic match
                returnVal = self.mock.mockReturnValues.get(self.name)
            if isinstance(returnVal, ReturnValuesBase):
                returnVal = returnVal.next()
            return returnVal

    def checkExpectations(self, thisCall, params, kwparams):
        if self.name in self.mock.mockExpectations:
            callsMade = len(self.mock.mockCalledMethods[self.name])
            for (expectation, after, until) in self.mock.mockExpectations[
                    self.name]:
                if callsMade > after and (until==0 or callsMade < until):
                    assert expectation(self.mock, thisCall, len(
                            self.mock.mockAllCalledMethods)-1), \
                            'Expectation failed: '+str(thisCall)


def _findFunc(cls, name):
    """ Depth first search for a method with a given name. """
    if cls.__dict__.has_key(name):
        return cls.__dict__[name]
    for base in cls.__bases__:
        func = _findFunc(base, name)
        if func:
            return func
    return None


class ReturnValuesBase:
    def next(self):
        try:
            return self.iter.next()
        except StopIteration:
            raise AssertionError("No more return values")
    def __iter__(self):
        return self

class ReturnValues(ReturnValuesBase):
    def __init__(self, *values):
        self.iter = iter(values)


class ReturnIterator(ReturnValuesBase):
    def __init__(self, iterator):
        self.iter = iter(iterator)


def expectParams(*params, **keywords):
    '''check that the callObj is called with specified params and keywords
    '''
    def fn(mockObj, callObj, idx):
        return callObj.args == params and callObj.kwargs == keywords
    return fn


def expectAfter(*methods):
    '''check that the function is only called after all the functions in 'methods'
    '''
    def fn(mockObj, callObj, idx):
        calledMethods = [method.name for method in mockObj.mockGetAllCalls()]
        #skip last entry, since that is the current call
        calledMethods = calledMethods[:-1]
        for method in methods:
            if method not in calledMethods:
                return False
        return True
    return fn

def expectException(exception, *args, **kwargs):
    ''' raise an exception when the method is called
    '''
    def fn(mockObj, callObj, idx):
        raise exception(*args, **kwargs)
    return fn


def expectParam(paramIdx, cond):
    '''check that the callObj is called with parameter specified by paramIdx (a
    position index or keyword) fulfills the condition specified by cond. cond is
    a function that takes a single argument, the value to test.
    '''
    def fn(mockObj, callObj, idx):
        param = callObj.getParam(paramIdx)
        return cond(param)
    return fn

def EQ(value):
    def testFn(param):
        return param == value
    return testFn

def NE(value):
    def testFn(param):
        return param != value
    return testFn

def GT(value):
    def testFn(param):
        return param > value
    return testFn

def LT(value):
    def testFn(param):
        return param < value
    return testFn

def GE(value):
    def testFn(param):
        return param >= value
    return testFn

def LE(value):
    def testFn(param):
        return param <= value
    return testFn

def AND(*condlist):
    def testFn(param):
        for cond in condlist:
            if not cond(param):
                return False
        return True
    return testFn

def OR(*condlist):
    def testFn(param):
        for cond in condlist:
            if cond(param):
                return True
        return False
    return testFn

def NOT(cond):
    def testFn(param):
        return not cond(param)
    return testFn

def MATCHES(regex, *args, **kwargs):
    compiled_regex = re.compile(regex, *args, **kwargs)
    def testFn(param):
        return compiled_regex.match(param) != None
    return testFn

def SEQ(*sequence):
    iterator = iter(sequence)
    def testFn(param):
        try:
            cond = iterator.next()
        except StopIteration:
            raise AssertionError('SEQ exhausted')
        return cond(param)
    return testFn

def IS(instance):
    def testFn(param):
        return param is instance
    return testFn

def ISINSTANCE(class_):
    def testFn(param):
        return isinstance(param, class_)
    return testFn

def ISSUBCLASS(class_):
    def testFn(param):
        return issubclass(param, class_)
    return testFn

def CONTAINS(val):
    def testFn(param):
        return val in param
    return testFn

def IN(container):
    def testFn(param):
        return param in container
    return testFn

def HASATTR(attr):
    def testFn(param):
        return hasattr(param, attr)
    return testFn

def HASMETHOD(method):
    def testFn(param):
        return hasattr(param, method) and callable(getattr(param, method))
    return testFn

CALLABLE = callable



from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt
from PyQt4.QtCore import *
from PyQt4.QtGui import *

def message_critical(title, text, detailed_text=None, informative_text=None,
    parent=None):
    """ Display a critical message in a L{QMessageBox}. """
    dlg = QMessageBox(QMessageBox.Critical, title, text, QMessageBox.Ok, parent)
    if informative_text:
        dlg.setInformativeText(informative_text)
    if detailed_text:
        dlg.setDetailedText(detailed_text)
    dlg.exec_()

def message_warning(title, text, detailed_text=None, informative_text=None,
    parent=None):
    """ Display a warning message in a L{QMessageBox}. """
    dlg = QMessageBox(QMessageBox.Warning, title, text, QMessageBox.Ok, parent)
    if informative_text:
        dlg.setInformativeText(informative_text)
    if detailed_text:
        dlg.setDetailedText(detailed_text)
    dlg.exec_()



""" Test QApplication wrapper. """
from srllib.qtgui import Application
from PyQt4.QtCore import QTimer

from _common import *

class ApplicationTest(TestCase):
    def test_set_excepthook(self):
        def exchook(exc, value, tb, threadname):
            self.__invoked = (exc, value, tb, threadname)
            
        def raiser():
            raise KeyboardInterrupt
        
        def quitter():
            Application.instance().quit()
            
        app = Application()
        app.sig_exception.connect(exchook)
        QTimer.singleShot(0, raiser)
        QTimer.singleShot(1, quitter)
        self.__invoked = None
        app.exec_()
        try: exc, val, tb = self.__invoked[:3]
        except TypeError:
            self.fail("Exception hook not called")
        if exc is not KeyboardInterrupt:
            import traceback; traceback.print_exception(exc, val, tb)
            self.fail("Unexpected exception type: %r" % exc)



""" Collection of Qt model classes.
"""
from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import Qt

class _AppendRowCommand(QtGui.QUndoCommand):
    __super = QtGui.QUndoCommand

    def __init__(self, model, items, text, parent=None):
        if text is None:
            text = "append row"
        self.__super.__init__(self, text, parent)

        self.__model, self.__items = model, items

    def redo(self):
        # The model takes over ownership of items, so hand it copies
        # Also make sure to clone, since the object may be of an arbitrary
        # QStandardItem subclass
        self.__model.appendRow([item.clone() for item in self.__items])

    def undo(self):
        self.__model.removeRow(self.__model.rowCount()-1)

class _SetDataCommand(QtGui.QUndoCommand):
    def __init__(self, undo_model, model, index, role2value, parent=None,
            clear=False):
        assert not isinstance(model, int)
        QtGui.QUndoCommand.__init__(self, "set item data")
        (self.__undo_model, self.__model, self.__row, self.__col,
                self.__role2value, self.__parent, self.__clear) = (undo_model,
                        model, index.row(), index.column(), role2value,
                        index.parent(), clear)
        self.__prev = undo_model.itemData(undo_model.mapFromSource(index))

    def redo(self):
        idx = self.__get_index()
        if self.__clear:
            # This clears all roles
            self.__model.setItemData(idx, {})
        for role, value in self.__role2value.items():
            self.__model.setData(idx, value, role)

    def undo(self):
        self.__model.setItemData(self.__get_index(), self.__prev)

    def __get_index(self):
        return self.__model.index(self.__row, self.__col, self.__parent)

class _RemoveRowCommand(QtGui.QUndoCommand):
    def __init__(self, model, row, parent):
        QtGui.QUndoCommand.__init__(self)
        self.__model, self.__row, self.__parent = model, row, parent

    def redo(self):
        self.__items = self.__model.takeRow(self.__row)

    def undo(self):
        self.__model.insertRow(self.__row, self.__items)

class _TakeItemCommand(QtGui.QUndoCommand):
    def __init__(self, model, row, column, text):
        if text is None:
            text = "take item"
        QtGui.QUndoCommand.__init__(self, text)

        self.item = None
        self.__model, self.__row, self.__column = model, row, column

    def redo(self):
        self.item = self.__model.takeItem(self.__row, self.__column)

    def undo(self):
        self.__model.setItem(self.__row, self.__column, self.item)
        self.item = None # The item is taken over by the model, access unsafe

class UndoItemModel(QtGui.QSortFilterProxyModel):
    """ Item model that is capable of undoing operations.

    This model implements undo support on top of QStandardItemModel, through a
    proxy scheme.

    QSortFilterProxyModel is subclassed since this makes it simple to implement
    a proxy (with undo logic) for a standard item model, in fact it is
    recommended by the Qt documentation to base proxy implementations on this.
    @ivar undo_stack: The undo stack.
    """
    __super = QtGui.QSortFilterProxyModel

    def __init__(self, undo_stack=None, hor_headers=None, ver_headers=None,
        parent=None):
        """ Constructor.
        @ivar undo_stack: Optionally pass an undo stack to work with. If None,
        the model instantiates its own.
        @ivar hor_headers: Optionally specify horizontal header labels.
        @ivar ver_headers: Optionally specify vertical header labels.
        @ivar parent: Optionally pass QObject parent.
        """
        self.__super.__init__(self, parent)

        model = self.__model = QtGui.QStandardItemModel(self)
        self.setSourceModel(model)
        if undo_stack is None:
            undo_stack = QtGui.QUndoStack(self)
        self.undo_stack = undo_stack

        if hor_headers:
            model.setHorizontalHeaderLabels(hor_headers)
        if ver_headers:
            model.setVerticalHeaderLabels(ver_headers)

    def append_row(self, values, undo_text=None):
        """ Append row of values.

        Values may be represented as dicts, as a mapping from role to value.
        Otherwise values are taken to be for Qt.EditRole.
        @param undo_text: Optionally specify undo text.
        """
        items = []
        for val in values:
            item = QtGui.QStandardItem()
            if not isinstance(val, dict):
                val = {Qt.EditRole: val}
            for role, data in val.items():
                item.setData(QtCore.QVariant(data), role)
            items.append(item)
        self.undo_stack.push(_AppendRowCommand(self.__model, items,
            text=undo_text))

    #{ Implement model interface

    def setData(self, index, value, role=Qt.EditRole):
        role2value = {role: value}
        self.undo_stack.push(_SetDataCommand(self, self.__model,
            self.__model.index(index.row(), index.column()), role2value))
        return True

    def setItemData(self, index, roles, clear=False):
        self.undo_stack.push(_SetDataCommand(self, self.__model,
            self.__model.index(index.row(), index.column()), roles, clear=clear))
        return True

    #}

    #{ Expose QStandardItemModel methods

    def item(self, row, column=0):
        return self.__model.item(row, column)

    def setItem(self, *args):
        self.__model.setItem(*args)

    def takeItem(self, row, column=0, undo_text=None):
        cmd = _TakeItemCommand(self.__model, row, column, text=undo_text)
        self.undo_stack.push(cmd)
        return cmd.item
        # return self.__model.takeItem(row, column)

    def appendRow(self, items, undo_text=None):
        """ Append row of L{items<QtGui.QStandardItem>}.
        @param undo_text: Optionally specify undo text.
        """
        self.undo_stack.push(_AppendRowCommand(self.__model, items, text=
            undo_text))

    def removeRow(self, row, parent=QtCore.QModelIndex()):
        self.undo_stack.push(_RemoveRowCommand(self.__model, row, parent))

    def removeRows(self, row, count, parent=QtCore.QModelIndex(), undo_text=
        None):
        if undo_text is None:
            undo_text = "remove rows"
        self.undo_stack.beginMacro(undo_text)
        try:
            for i in reversed(range(row, row+count)):
                self.removeRow(i, parent)
        finally:
            self.undo_stack.endMacro()

    def setHorizontalHeaderLabels(self, labels):
        self.__model.setHorizontalHeaderLabels(labels)

    def setColumnCount(self, count):
        self.__model.setColumnCount(count)

    def columnCount(self, parent=QtCore.QModelIndex()):
        return self.__model.columnCount()

    #}
